@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -51,11 +51,12 @@
   {
     int (*init) (gnutls_mac_algorithm_t, void **ctx);
     int (*setkey) (void *ctx, const void *key, size_t keysize);
-    void (*reset) (void *ctx);
+    int (*setnonce) (void *ctx, const void *nonce, size_t noncesize);
     int (*hash) (void *ctx, const void *text, size_t textsize);
     int (*output) (void *src_ctx, void *digest, size_t digestsize);
     void (*deinit) (void *ctx);
-    int (*fast)(gnutls_mac_algorithm_t, const void *key, size_t keysize, const void *text, size_t textsize, void *digest);
+    int (*fast)(gnutls_mac_algorithm_t, const void* nonce, size_t nonce_size,
+    	const void *key, size_t keysize, const void *text, size_t textsize, void *digest);
 
     /* Not needed for registered on run-time. Only included
      * should define it. */
@@ -65,7 +66,6 @@
   typedef struct
   {
     int (*init) (gnutls_digest_algorithm_t, void **ctx);
-    void (*reset) (void *ctx);
     int (*hash) (void *ctx, const void *src, size_t srcsize);
     int (*output) (void *src_ctx, void *digest, size_t digestsize);
     void (*deinit) (void *ctx);
@@ -80,6 +80,7 @@
   {
     int (*init) (void **ctx);
     int (*rnd) (void *ctx, int level, void *data, size_t datasize);
+    void (*rnd_refresh) (void *ctx);
     void (*deinit) (void *ctx);
   } gnutls_crypto_rnd_st;
 
@@ -116,6 +117,7 @@
   {
     bigint_t (*bigint_new) (int nbits);
     void (*bigint_release) (bigint_t n);
+    void (*bigint_clear) (bigint_t n); /* zeros the int */
     /* 0 for equality, > 0 for m1>m2, < 0 for m1<m2 */
     int (*bigint_cmp) (const bigint_t m1, const bigint_t m2);
     /* as bigint_cmp */
@@ -193,6 +195,7 @@
 
 
   void gnutls_pk_params_release (gnutls_pk_params_st * p);
+  void gnutls_pk_params_clear (gnutls_pk_params_st * p);
   void gnutls_pk_params_init (gnutls_pk_params_st * p);
 
 
@@ -201,7 +204,7 @@
 /* parameters should not be larger than this limit */
 #define DSA_PUBLIC_PARAMS 4
 #define RSA_PUBLIC_PARAMS 2
-#define ECC_PUBLIC_PARAMS 8
+#define ECC_PUBLIC_PARAMS 2
 
 
 #define MAX_PRIV_PARAMS_SIZE GNUTLS_MAX_PK_PARAMS       /* ok for RSA and DSA */
@@ -209,7 +212,7 @@
 /* parameters should not be larger than this limit */
 #define DSA_PRIVATE_PARAMS 5
 #define RSA_PRIVATE_PARAMS 8
-#define ECC_PRIVATE_PARAMS 9
+#define ECC_PRIVATE_PARAMS 3
 
 #if MAX_PRIV_PARAMS_SIZE - RSA_PRIVATE_PARAMS < 0
 #error INCREASE MAX_PRIV_PARAMS
@@ -257,15 +260,9 @@
  *  [8] is k (private key)
  */
 
-#define ECC_PRIME 0
-#define ECC_ORDER 1
-#define ECC_A 2
-#define ECC_B 3
-#define ECC_GX 4
-#define ECC_GY 5
-#define ECC_X 6
-#define ECC_Y 7
-#define ECC_K 8
+#define ECC_X 0
+#define ECC_Y 1
+#define ECC_K 2
 
 #define DSA_P 0
 #define DSA_Q 1
