@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2009-2012 Free Software Foundation, Inc.
+ * Copyright (C) 2013 Nikos Mavrogiannopoulos
  *
  * Authors: Jonathan Bastien-Filiatrault
  *          Nikos Mavrogiannopoulos
@@ -8,7 +9,7 @@
  *
  * The GNUTLS library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -34,17 +35,8 @@
 #include <gnutls_constate.h>
 #include <gnutls_state.h>
 #include <gnutls/dtls.h>
-#include <timespec.h>
 #include <algorithms.h>
 
-/* returns a-b in ms */
-unsigned int
-_dtls_timespec_sub_ms (struct timespec *a, struct timespec *b)
-{
-  return (a->tv_sec * 1000 + a->tv_nsec / (1000 * 1000) -
-          (b->tv_sec * 1000 + b->tv_nsec / (1000 * 1000)));
-}
-
 void
 _dtls_async_timer_delete (gnutls_session_t session)
 {
@@ -247,7 +239,7 @@ unsigned int timeout;
             {
               /* if no retransmission is required yet just return 
                */
-              if (_dtls_timespec_sub_ms(&now, &session->internals.dtls.last_retransmit) < TIMER_WINDOW)
+              if (timespec_sub_ms(&now, &session->internals.dtls.last_retransmit) < TIMER_WINDOW)
                 {
                   gnutls_assert();
                   goto nb_timeout;
@@ -278,7 +270,7 @@ unsigned int timeout;
     {
       timeout = TIMER_WINDOW;
 
-      diff = _dtls_timespec_sub_ms(&now, &session->internals.dtls.handshake_start_time);
+      diff = timespec_sub_ms(&now, &session->internals.dtls.handshake_start_time);
       if (diff >= session->internals.dtls.total_timeout_ms) 
         {
           _gnutls_dtls_log("Session timeout: %u ms\n", diff);
@@ -286,7 +278,7 @@ unsigned int timeout;
           goto end_flight;
         }
 
-      diff = _dtls_timespec_sub_ms(&now, &session->internals.dtls.last_retransmit);
+      diff = timespec_sub_ms(&now, &session->internals.dtls.last_retransmit);
       if (session->internals.dtls.flight_init == 0 || diff >= TIMER_WINDOW)
         {
           _gnutls_dtls_log ("DTLS[%p]: %sStart of flight transmission.\n", session,  (session->internals.dtls.flight_init == 0)?"":"re-");
@@ -435,21 +427,28 @@ int ret;
   return 0;
 }
 
-
 #define window_table rp->record_sw
 #define window_size rp->record_sw_size
+#define window_head_idx rp->record_sw_head_idx
 
-/* FIXME: could we modify that code to avoid using
- * uint64_t?
- */
-
-static void rot_window(struct record_parameters_st * rp, int places)
+static void slide_window(struct record_parameters_st * rp, unsigned int places)
 {
-  window_size -= places;
-  memmove(window_table, &window_table[places], window_size*sizeof(window_table[0]));
+unsigned int old_head = window_head_idx;
+
+  if (places < window_size)
+    {
+      window_head_idx += places;
+      window_head_idx %= DTLS_RECORD_WINDOW_SIZE;
+      
+      window_table[window_head_idx] = window_table[old_head] + places;
+    }
+  else
+    {
+      unsigned int last_idx = (window_head_idx + window_size - 1) % window_size;
+      window_table[window_head_idx] = window_table[last_idx];
+    }
 }
 
-#define MOVE_SIZE 20
 /* Checks if a sequence number is not replayed. If replayed
  * returns a negative error code, otherwise zero.
  */
@@ -457,6 +456,7 @@ int _dtls_record_check(struct record_parameters_st *rp, uint64 * _seq)
 {
 uint64_t seq = 0, diff;
 unsigned int i, offset = 0;
+unsigned int last_idx;
 
   for (i=2;i<8;i++)
     {
@@ -464,68 +464,71 @@ unsigned int i, offset = 0;
       seq |= _seq->i[i] & 0xff;
     }
 
+  /* only two values allowed in window_size */
   if (window_size == 0)
     {
       window_size = 1;
-      window_table[0] = seq;
+      window_head_idx = 0;
+      last_idx = window_size - 1;
+      window_table[last_idx] = window_table[window_head_idx] = seq;
       return 0;
     }
 
-  if (seq <= window_table[0])
+  last_idx = (window_head_idx + window_size - 1) % window_size;
+
+  if (seq <= window_table[window_head_idx])
     {
       return -1;
     }
 
-  if (window_size == DTLS_RECORD_WINDOW_SIZE) 
-    {
-      rot_window(rp, MOVE_SIZE);
-    }
-
-  if (seq < window_table[window_size-1])
+  if (seq <= window_table[last_idx])
     {
       /* is between first and last */
-      diff = window_table[window_size-1] - seq;
+      diff = window_table[last_idx] - seq;
 
       if (diff >= window_size) 
-        return -1;
-
-      offset = window_size-1-diff;
-
-      if (window_table[offset] == seq)
-        return -1;
-      else
-        window_table[offset] = seq;
-    }
-  else /* seq >= last */
-    {
-      if (seq == window_table[window_size-1]) 
         {
           return -1;
         }
 
-      diff = seq - window_table[window_size-1];
-      if (diff <= DTLS_RECORD_WINDOW_SIZE - window_size)
-        { /* fits in our empty space */
-          offset = diff + window_size-1;
+      if (diff > last_idx)
+        {
+          diff = diff - last_idx;
+          offset = window_size - 1 - diff;
+        }
+      else
+        offset = last_idx - diff;
 
-          window_table[offset] = seq;
-          window_size = offset + 1;
+      if (window_table[offset] == seq)
+        {
+          return -1;
+        }
+      else
+        window_table[offset] = seq;
+    }
+  else /* seq > last */
+    {
+      diff = seq - window_table[last_idx];
+
+      if (window_size + diff <= DTLS_RECORD_WINDOW_SIZE)
+        {
+          window_size += diff;
         }
       else
         {
-          if (diff > DTLS_RECORD_WINDOW_SIZE/2)
-            { /* difference is too big */
-              window_table[DTLS_RECORD_WINDOW_SIZE-1] = seq;
-              window_size = DTLS_RECORD_WINDOW_SIZE;
-            }
-          else
+          if (window_size < DTLS_RECORD_WINDOW_SIZE)
             {
-              rot_window(rp, diff);
-              offset = diff + window_size-1;
-              window_table[offset] = seq;
-              window_size = offset + 1;            
+              offset = DTLS_RECORD_WINDOW_SIZE-window_size;
+              window_size = DTLS_RECORD_WINDOW_SIZE;
+              diff -= offset;
             }
+          
+	  /* diff > 0 */
+          slide_window(rp, diff);
         }
+
+      offset = (window_head_idx + window_size - 1) % window_size;
+      window_table[offset] = seq;
     }
   return 0;
 }
@@ -598,27 +601,36 @@ int total = 0, ret, iv_size;
     return gnutls_assert_val(ret);
 
   /* requires padding */
-  iv_size = _gnutls_cipher_get_iv_size(params->cipher_algorithm);
 
   if (_gnutls_cipher_is_block (params->cipher_algorithm) == CIPHER_BLOCK)
     {
-      *blocksize = iv_size;
+      iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
+      total += iv_size;
 
-      total += iv_size; /* iv_size == block_size in DTLS */
+      *blocksize = gnutls_cipher_get_block_size(params->cipher_algorithm);
 
       /* We always pad with at least one byte; never 0. */
-      total++;
+      if (session->security_parameters.new_record_padding == 0)
+        total++;
     }
   else
     {
       *blocksize = 1;
     }
+
+  if (session->security_parameters.new_record_padding != 0)
+    total += 2;
   
   if (params->mac_algorithm == GNUTLS_MAC_AEAD)
-    total += _gnutls_cipher_get_tag_size(params->cipher_algorithm);
+    {
+      iv_size = gnutls_cipher_get_iv_size(params->cipher_algorithm);
+      total += iv_size;
+
+      total += _gnutls_cipher_get_tag_size(params->cipher_algorithm);
+    }
   else
     {
-      ret = _gnutls_hmac_get_algo_len(params->mac_algorithm);
+      ret = _gnutls_mac_get_algo_len(params->mac_algorithm);
       if (ret < 0)
         return gnutls_assert_val(ret);
       total+=ret;
@@ -636,7 +648,7 @@ int total = 0, ret, iv_size;
  *
  * This function will return the actual maximum transfer unit for
  * application data. I.e. DTLS headers are subtracted from the
- * actual MTU.
+ * actual MTU which is set using gnutls_dtls_set_mtu().
  *
  * Returns: the maximum allowed transfer unit.
  *
@@ -673,6 +685,9 @@ int overhead;
  * renegotiation, in order to ensure that the MTU is still sufficient to
  * account for the new protocol overhead.
  *
+ * In most cases you only need to call gnutls_dtls_set_mtu() with
+ * the maximum MTU of your transport layer.
+ *
  * Returns: %GNUTLS_E_SUCCESS (0) on success, or a negative error code.
  *
  * Since: 3.1
@@ -690,8 +705,11 @@ int gnutls_dtls_set_data_mtu (gnutls_session_t session, unsigned int mtu)
   mtu += overhead;
 
   /* Round it up to the next multiple of blocksize */
-  mtu += blocksize - 1;
-  mtu -= mtu % blocksize;
+  if (blocksize > 1) 
+    {
+      mtu += blocksize - 1;
+      mtu -= mtu % blocksize;
+    }
 
   /* Add the *unencrypted header size */
   mtu += RECORD_HEADER_SIZE(session);
@@ -739,7 +757,7 @@ unsigned int diff;
 
   gettime(&now);
   
-  diff = _dtls_timespec_sub_ms(&now, &session->internals.dtls.last_retransmit);
+  diff = timespec_sub_ms(&now, &session->internals.dtls.last_retransmit);
   if (diff >= TIMER_WINDOW)
     return 0;
   else
@@ -774,6 +792,10 @@ unsigned int diff;
  * for (i.e. #gnutls_session_t) is the one that the 
  * original incoming packet was originated from.
  *
+ * This function must be called at the first incoming packet,
+ * prior to allocating any resources and must be succeeded
+ * by gnutls_dtls_cookie_verify().
+ *
  * Returns: the number of bytes sent, or a negative error code.  
  *
  * Since: 3.0
@@ -848,7 +870,7 @@ uint8_t digest[C_HASH_SIZE];
   hvr[hvr_size++] = 255;
   hvr[hvr_size++] = COOKIE_SIZE;
 
-  ret = _gnutls_hmac_fast(C_HASH, key->data, key->size, client_data, client_data_size, digest);
+  ret = _gnutls_mac_fast(C_HASH, key->data, key->size, client_data, client_data_size, digest);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -871,11 +893,13 @@ uint8_t digest[C_HASH_SIZE];
  * @msg_size: The size of the message.
  * @prestate: The cookie of this client.
  *
- * This function will verify an incoming message for
+ * This function will verify the received message for
  * a valid cookie. If a valid cookie is returned then
  * it should be associated with the session using
  * gnutls_dtls_prestate_set();
  *
+ * This function must be called after gnutls_dtls_cookie_send().
+ *
  * Returns: %GNUTLS_E_SUCCESS (0) on success, or a negative error code.  
  *
  * Since: 3.0
@@ -923,7 +947,7 @@ uint8_t digest[C_HASH_SIZE];
       return gnutls_assert_val(GNUTLS_E_BAD_COOKIE);
     }
 
-  ret = _gnutls_hmac_fast(C_HASH, key->data, key->size, client_data, client_data_size, digest);
+  ret = _gnutls_mac_fast(C_HASH, key->data, key->size, client_data, client_data_size, digest);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
@@ -946,6 +970,9 @@ uint8_t digest[C_HASH_SIZE];
  * the cookie authentication with the client, with the newly 
  * established session.
  *
+ * This functions must be called after a successful gnutls_dtls_cookie_verify()
+ * and should be succeeded by the actual DTLS handshake using gnutls_handshake().
+ *
  * Since: 3.0
  **/
 void gnutls_dtls_prestate_set(gnutls_session_t session, gnutls_dtls_prestate_st* prestate)
