@@ -25,7 +25,6 @@
 #include <gnutls_datum.h>
 #include <gnutls_global.h>
 #include <gnutls_errors.h>
-#include <gnutls_rsa_export.h>
 #include <common.h>
 #include <gnutls_x509.h>
 #include <x509_b64.h>
@@ -825,7 +824,6 @@ read_pkcs_schema_params (schema_id * schema, const char *password,
     case PKCS12_3DES_SHA1:
     case PKCS12_ARCFOUR_SHA1:
     case PKCS12_RC2_40_SHA1:
-      memset(enc_params, 0, sizeof(*enc_params));
 
       if ((*schema) == PKCS12_3DES_SHA1)
         {
@@ -873,7 +871,7 @@ read_pkcs_schema_params (schema_id * schema, const char *password,
       if (enc_params->iv_size)
         {
           result =
-            _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 2 /*IV*/, kdf_params->salt,
+            _gnutls_pkcs12_string_to_key (2 /*IV*/, kdf_params->salt,
                                           kdf_params->salt_size,
                                           kdf_params->iter_count, password,
                                           enc_params->iv_size,
@@ -1767,7 +1765,7 @@ decrypt_data (schema_id schema, ASN1_TYPE pkcs8_asn,
       break;
     default:
       result =
-        _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 1 /*KEY*/, kdf_params->salt,
+        _gnutls_pkcs12_string_to_key (1 /*KEY*/, kdf_params->salt,
                                       kdf_params->salt_size,
                                       kdf_params->iter_count, password,
                                       key_size, key);
@@ -2071,7 +2069,7 @@ generate_key (schema_id schema,
   key->size = kdf_params->key_size =
     gnutls_cipher_get_key_size (enc_params->cipher);
 
-  enc_params->iv_size = _gnutls_cipher_get_iv_size (enc_params->cipher);
+  enc_params->iv_size = gnutls_cipher_get_iv_size (enc_params->cipher);
   key->data = gnutls_malloc (key->size);
   if (key->data == NULL)
     {
@@ -2113,7 +2111,7 @@ generate_key (schema_id schema,
 
     default:
       ret =
-        _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 1 /*KEY*/, kdf_params->salt,
+        _gnutls_pkcs12_string_to_key (1 /*KEY*/, kdf_params->salt,
                                       kdf_params->salt_size,
                                       kdf_params->iter_count, password,
                                       kdf_params->key_size, key->data);
@@ -2128,7 +2126,7 @@ generate_key (schema_id schema,
       if (enc_params->iv_size)
         {
           ret =
-            _gnutls_pkcs12_string_to_key (GNUTLS_MAC_SHA1, 2 /*IV*/, kdf_params->salt,
+            _gnutls_pkcs12_string_to_key (2 /*IV*/, kdf_params->salt,
                                           kdf_params->salt_size,
                                           kdf_params->iter_count, password,
                                           enc_params->iv_size,
